(document:surround "/std/frame")

(define squashfs-list
  '("fast" "normal" "tight"))

(gridbox columns "10;40;40;10"
	 (spacer)
	 (document:id targets
	   (listbox name "target"
		    header (_ "Target image")
		  (when changed
		    (allow-builds #t #f))
		  (when selected
		    (let ((t (not-empty-string? (form-value "target"))))
		      (test-button activity t)
		      (if t
			(status-set (_ "Let's test it before building"))
			(choose-target))))))
	 (gridbox columns "20;80"
		  ;;
		  (hbox colspan 2
			(label text (bold (_ "Image configuration:")) align "center"))
		  ;;
;;; ARCH needs apt-conf support; let's assume native one for a while
;		  (document:id ARCH
;			       (combobox name "ARCH"
;					 enumref "/mkimage/arches"))
;		  (label (_ "Architecture") align "left")
		  ;;
		  (document:id SQUASHFS
			       (combobox name "SQUASHFS" rows squashfs-list))
		  (label (_ "SquashFS compression") align "left")
		  ;;
		  (document:id ISOHYBRID
			       (checkbox name "ISOHYBRID" align "right"))
		  (label (_ "Hybrid ISO") align "left")
		  ;;
		  (hbox colspan 2 (separator))
		  ;;
		  (document:id reset-button
			       (button (_ "Reset") align "center"))
		  (hbox
		    (document:id test-button (button (_ "Test")))
		    (document:id build-button (button (_ "Build"))))
		  (spacer))
	 (spacer)
	 ;;
	 (spacer)
	 (document:id progress (progressbar))
	 (document:id status-label (label))
	 (spacer)
	 ;;
	 (spacer)
	 (hbox colspan 2
	       (document:id result (edit alterability #f)))
	 (spacer))

;;;;;;;;;;;;;;;;;;;;;; interface handling

(define (status-set message)
  (status-label text (bold message)))

;;; tweak availability/behaviour of the form in different states
(define (status message inter active)
  (status-set message)
;  (format (current-error-port) "status: ~S~%" message)
  (allow-changes inter)
  (if active
    (progress minimum 0 value 0 maximum 0)
    (progress minimum 0 value 0 maximum -1 text (_ "idle"))))

;;; process incoming messages from backend
(define (message-handler args)
  (format (current-error-port) "message-handler: ~S~%" args)
  (and-let* ((msg (cond-plistq 'message args #f)))
	    (cond
	      ;;status
	      ((string-match "root builds unsupported" msg)
	       (status (_ "Builds using root privileges are not supported.") #f #f))
;;; can be post- or pre-, and is generally pretty fast
;	      ((string-match "cleaning up after build" msg)
;	       #t)
;	      ((string-match "cleaning up" msg)
;	       (status-set (_ "Cleaning up")))
	      ((string-match "initializing BUILDDIR" msg)
	       (status (_ "Initializing build directory...") #f #t))
	      ((string-match "preparing distro config" msg)
	       (status (_ "Preparing image configuration...") #f #t))
	      ((string-match "starting image build" msg)
	       (status (_ "Building image, please wait...") #f #t))
	      ((string-match "done" msg)
	       (status (_ "Done.") #t #f))
	      ((string-match "failed" msg)
	       (status (_ "Failed!") #t #f))
	      ((string-match ".  Stop." msg)
	       (begin
		 (status (_ "Failed!") #t #f)
		 (allow-builds #f #f)
	         (result text msg)))
	      ((string-match "skipping actual image build" msg)
	       (begin
		 (status (_ "Test passed") #t #f)
		 (result text "(no actual image yet)")))
	      ((string-match "image: " msg)
	       (begin
		 (status "Image successfully built:" #t #f)
		 (result text (substring msg 7))))
	      )))

;;; FIXME: how to hook it up for pre-exit cleanup actions?
;(define (stop)
;  (remove-mailbox message-handler))

(define (allow-builds test build)
  (test-button activity test)
  (build-button activity build))

(define (allow-changes v)
  (allow-builds v v)
  (targets activity v)
;  (ARCH activity v)
  (SQUASHFS activity v)
  (ISOHYBRID activity v)
  (reset-button activity v))

(define (choose-target)
  (status-set "Choose target"))

;;;;;;;;;;;;;;;;;;;;;; backend communication

(define (reset-interface)
  (catch/message
    (lambda()
      (begin
	(form-update-enum "target" (woo-list "/mkimage/targets"))
	(ISOHYBRID state #t)
	(SQUASHFS current 1)
;       (ARCH current 0)
	(targets activity #t)
	(choose-target)
	(allow-builds #f #f)
	))))

;;; "not really" means CHECK=1
(define (run-build really)
  (begin
    (allow-changes #f)
    (catch/message
      (lambda()
	(woo-write "/mkimage"
		   'target (form-value "target")
;		   'ARCH (ARCH text)
		   'SQUASHFS (SQUASHFS text)
		   'ISOHYBRID (form-value "ISOHYBRID")
		   'CHECK (not really)
		   )))))

;;;;;;;;;;;;;;;;;;;;;; button handlers

(test-button
  (when clicked (run-build #f)))

(build-button
  (when clicked (run-build #t)))

(reset-button
  (when clicked (reset-interface)))

;;;;;;;;;;;;;;;;;;;;;; initialization
(document:root
  (when loaded
    (reset-interface)
    (add-mailbox message-handler)))
